<?php
/*
Plugin Name: KWHero WordPress Integration
Plugin URI: https://kwhero.com/
Description: KWHero - Perfect SEO Content to Skyrocket Your Organic Search Traffic. This plugin allows you to publish content to your website straight from your KWHero dashboard.
Author: KWHero
Author URI: https://kwhero.com/
Version: 1.0.4
*/

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

define('KWH_PATH', plugin_dir_path(__FILE__));
define('KWH_URL', plugin_dir_url(__FILE__));
define('KWH_CURRENT_VERSION', '1.0.4');
define('KWH_API_BASE', 'https://kwhero.com/wp-json/kwh-plugin/v1/');

include_once(KWH_PATH . 'update.php');

// register publish content end point
add_action( 'rest_api_init', function () {
    register_rest_route( 'kwhero/v1', 'post/publish', array(
      'methods' => 'POST',
      'callback' => 'kwh_publish_content',
    ) );
  } );

// create or update a post 
function kwh_publish_content( WP_REST_Request $request ) {
    // get options
    $apiKey = $request['apiKey'];
    $post_id = $request['post_id'];
    $title = $request['title'];
    $content = $request['content'];
    $post_status = $request['post_status'];
    $categories = $request['categories'];
  
    // Use post_date_gmt instead of post_date
    $post_date_gmt_str = $request['post_date_gmt'];
  
    // check if the date format is correct
    $date_format = 'Y-m-d H:i:s';
    $d = DateTime::createFromFormat($date_format, $post_date_gmt_str);
  
    // check if date could be processed
    if (!$d) {
        return new WP_Error( 'invalid_date', 'Date cannot be processed', array('status' => 400));
    }
  
    $post_date_gmt = date_format($d, 'Y-m-d H:i:s');
  
    // check if api is correct, return if mismatch
    $savedApiKey = get_option( 'kwhero_api_key' );
    if( $savedApiKey != $apiKey ) {
        $data = array( 'success' => false, 'message' => 'Invalid API key', 'status' => 400 );
  
        $response = new WP_REST_Response( $data );
        return $response;
    }
  
    // Get the first administrator user to set as an author
    $admin_args = array(
        'role'    => 'administrator',
        'orderby' => 'ID',
        'order'   => 'ASC',
        'number'  => 1,
    );
    $admins = get_users($admin_args);
  
    // Check if we have an admin user
    if (!empty($admins)) {
        $wp_admin = $admins[0]->ID;
    } else {
        $wp_admin = 1;
    }
  
    // create post if no post id is provided
    if (!$post_id){
        $args = array(
            'post_title'    => $title,
            'post_content'  => $content,
            'post_category' => $categories,
            'post_status'   => $post_status,
            'post_author'   => $wp_admin,
            'meta_input'  => array('kwhero_post' => true)
        );
  
        // handle scheduled posts
        if( in_array($post_status, array( 'future', 'draft', 'pending')) ) {
            // Replace post_date with post_date_gmt
            $args['post_date_gmt'] = $post_date_gmt;
        }
  
        $post_id = wp_insert_post($args);
        if(!is_wp_error($post_id)){
  
            $data = array( 'success' => true, 'post_id' => $post_id, 'status' => 200 );
  
            $response = new WP_REST_Response( $data );
            return $response;
        } else{
  
            $data = array( 'success' => false, 'message' => 'WP insert post error', 'status' => 400 );
  
            $response = new WP_REST_Response( $data );
            return $response;
        }
    } else {
        // update post if post id is provided
        $args = array(
            'ID'            => $post_id,
            'post_title'    => $title,
            'post_content'  => $content,
            'post_category' => $categories,
            'post_status'   => $post_status,
            'post_author'   => $wp_admin,
            'meta_input'  => array('kwhero_post' => true)
        );
  
        // handle scheduled posts
        if( in_array($post_status, array( 'future', 'draft', 'pending')) ) {
            $args['post_date_gmt'] = $post_date_gmt;
        }
  
        // update the post into the database
        $post_id = wp_update_post($args);
        if(!is_wp_error($post_id)){
  
            $data = array( 'success' => true, 'post_id' => $post_id, 'status' => 200 );
            // Create the response object
            $response = new WP_REST_Response( $data );
            return $response;
        } else{
            //there was an error in the post update 
            $data = array( 'success' => false, 'message' => 'WP update post error', 'status' => 400 );
            // Create the response object
            $response = new WP_REST_Response( $data );
            return $response;
        }
    }
  }
// register end point to get all post titles and ids
add_action( 'rest_api_init', function () {
    register_rest_route( 'kwhero/v1', 'posts', array(
      'methods' => 'GET',
      'callback' => 'get_post_titles_ids',
    ) );
  } );

// get all post titles and ids
function get_post_titles_ids( WP_REST_Request $request ) {
    // get options
    $apiKey = $request['apiKey'];

    //  check if api is correct, return if mismatch
    $savedApiKey = get_option( 'kwhero_api_key' );
    if( $savedApiKey != $apiKey ) {
        $data = array( 'success' => false, 'message' => 'Invalid API key', 'status' => 400 );
        // Create the response object
        $response = new WP_REST_Response( $data );
        return $response;
    }

    $args = array(
        'posts_per_page'   => -1,
        'orderby'          => 'id',
        'order'            => 'ASC',
        'post_type'        => 'post',
        'post_status'      => array('publish', 'draft', 'pending', 'future'),
        'suppress_filters' => true 
    );
    $posts_array = get_posts( $args );
    $posts = array();
    foreach ($posts_array as $post) {
        $posts[] = array(
            'id' => $post->ID,
            'title' => $post->post_title
        );
    }
    $data = array( 'success' => true, 'posts' => $posts, 'status' => 200 );
    // Create the response object
    $response = new WP_REST_Response( $data );
    return $response;
}

// register end point to get all categories name and ids and parent ids
add_action( 'rest_api_init', function () {
    register_rest_route( 'kwhero/v1', 'categories', array(
      'methods' => 'GET',
      'callback' => 'get_category_names_ids',
    ) );
  } );

// get all categories name and ids and parent ids
function get_category_names_ids( WP_REST_Request $request ) {
    // get options
    $apiKey = $request['apiKey'];

    //  check if api is correct, return if mismatch
    $savedApiKey = get_option( 'kwhero_api_key' );
    if( $savedApiKey != $apiKey ) {
        $data = array( 'success' => false, 'message' => 'Invalid API key', 'status' => 400 );
        // Create the response object
        $response = new WP_REST_Response( $data );
        return $response;
    }

    $args = array(
        'orderby' => 'name',
        'order' => 'ASC',
        'hide_empty' => false,
        'number' => '',
        'fields' => 'all',
        'hierarchical' => true,
        'pad_counts' => false,
        'cache_domain' => 'core'
    );
    $categories = get_categories( $args );
    $categories_array = array();
    foreach ($categories as $category) {
        $categories_array[] = array(
            'id' => $category->term_id,
            'name' => $category->name,
            'parent_id' => $category->parent
        );
    }
    $data = array( 'success' => true, 'categories' => $categories_array, 'status' => 200 );
    // Create the response object
    $response = new WP_REST_Response( $data );
    return $response;
}

// register an end point to get post categories by post id
add_action( 'rest_api_init', function () {
    register_rest_route( 'kwhero/v1', 'post/getcategories', array(
      'methods' => 'GET',
      'callback' => 'get_post_categories',
    ) );
  } );

// get post categories by post id
function get_post_categories( WP_REST_Request $request ) {
    // get options
    $apiKey = $request['apiKey'];
    $post_id = $request['post_id'];

    //  check if api is correct, return if mismatch
    $savedApiKey = get_option( 'kwhero_api_key' );
    if( $savedApiKey != $apiKey ) {
        $data = array( 'success' => false, 'message' => 'Invalid API key', 'status' => 400 );
        // Create the response object
        $response = new WP_REST_Response( $data );
        return $response;
    }
    // check if post exists
    if(get_post_status($post_id)){
        $categories = get_the_category( $post_id );
        $categories_array = array();
        foreach ($categories as $category) {
            $categories_array[] = array(
                'id' => $category->term_id,
                'name' => $category->name,
                'parent_id' => $category->parent
            );
        }
        $data = array( 'success' => true, 'categories' => $categories_array, 'status' => 200 );
        // Create the response object
        $response = new WP_REST_Response( $data );
        return $response;
    } else {
        $data = array( 'success' => false, 'message' => 'Invalid post id', 'status' => 400 );
        // Create the response object
        $response = new WP_REST_Response( $data );
        return $response;
    }
    
}


// register an end point to get niterlinking data
add_action( 'rest_api_init', function () {
    register_rest_route( 'kwhero/v1', 'interlinking-data', array(
      'methods' => 'POST',
      'callback' => 'get_interlinking_data',
    ) );
  } );

// get post categories by post id
function get_interlinking_data( WP_REST_Request $request ) {
    // Retrieve parameters from request body
    $body = $request->get_json_params();
    $apiKey = isset($body['apiKey']) ? $body['apiKey'] : '';
    $type = isset($body['type']) && !empty($body['type']) ? $body['type'] : 'both';

    // Validate API Key
    $savedApiKey = get_option('kwhero_api_key');
    if ($savedApiKey != $apiKey) {
        return new WP_REST_Response(array('success' => false, 'message' => 'Invalid API key'), 403);
    }

    // Validate 'type' parameter
    if (!in_array($type, array('post', 'page', 'both'))) {
        return new WP_REST_Response(array('success' => false, 'message' => 'Invalid type parameter'), 400);
    }

    // Define the query based on the type
    $args = array(
        'post_type' => $type === 'both' ? array('post', 'page') : array($type),
        'post_status' => array('publish', 'draft'),
        'posts_per_page' => 1000, // Limit to 1000 rows as specified
    );

    $query = new WP_Query($args);
    $data = array();

    // Loop through posts/pages and construct the response data
    if ($query->have_posts()) {
        while ($query->have_posts()) {
            $query->the_post();
            $data[] = array(
                'title' => get_the_title(),
                'url' => get_permalink(),
                'status' => get_post_status(),
            );
        }
    }

    wp_reset_postdata();

    // Return the response
    return new WP_REST_Response(array('success' => true, 'data' => $data), 200);
}



// donwload images locally
add_action('save_post', 'download_kwhero_images', 10, 3);

function download_kwhero_images($post_id, $post, $update) {
    $kwhero_post = get_post_meta($post_id, 'kwhero_post', true);

    if ($kwhero_post) {
        $content = $post->post_content;
        //error_log("it is a kwh post \n", 3, ABSPATH . 'error.log');

        preg_match_all('/<img[^>]+src="([^">]+)"/', $content, $matches);
        $images = $matches[1];

        foreach ($images as $src) {
            $host = parse_url($src, PHP_URL_HOST);
            //error_log("image host is $host \n", 3, ABSPATH . 'error.log');

            if ($host == 'www.kwhero.com' || $host == 'kwhero.com') {
                // Download the image and save it as an attachment to the post
                require_once(ABSPATH . 'wp-admin/includes/media.php');
                require_once(ABSPATH . 'wp-admin/includes/file.php');
                require_once(ABSPATH . 'wp-admin/includes/image.php');
                
                //error_log("required needed files \n", 3, ABSPATH . 'error.log');

                $sideloaded_image = media_sideload_image($src, $post_id);
                //error_log("sideload_image  $sideloaded_image \n", 3, ABSPATH . 'error.log');

                if (is_wp_error($sideloaded_image)) {
                    //error_log("Error sideloading image $src: " . $sideloaded_image->get_error_message() . "\n", 3, ABSPATH . 'error.log');
                    continue;
                }

                $attachments = get_posts(array(
                    'numberposts' => '1',
                    'post_parent' => $post_id,
                    'post_type' => 'attachment',
                    'post_mime_type' => 'image',
                    'order' => 'DESC'
                ));
                
                //$sizeof = sizeof($attachments);
                //error_log("size of attach  $sizeof \n", 3, ABSPATH . 'error.log');
                
                if (sizeof($attachments) > 0) {
                    // Replace the old image URL with the new one
                    $new_src = wp_get_attachment_url($attachments[0]->ID);
                    //error_log("old src  $src \n", 3, ABSPATH . 'error.log');
                    //error_log("new src  $new_src \n", 3, ABSPATH . 'error.log');
                    $content = str_replace($src, $new_src, $content);
                    //error_log("replaced content $content \n", 3, ABSPATH . 'error.log');
                }
            }
        }

        // Remove filter to avoid loop
        remove_action('save_post', 'download_kwhero_images');
        $updated_post = wp_update_post(array(
            'ID' => $post_id,
            'post_content' => $content,
        ));
        if (is_wp_error($updated_post)) {
            error_log("Error updating post $post_id: " . $updated_post->get_error_message() . "\n", 3, ABSPATH . 'error.log');
        }
        // Add the function back
        add_action('save_post', 'download_kwhero_images', 10, 3);
    }
}





// Settings Page: KWHero
// generated by https://wp-skills.com/tools/settings-options-page-generator
// Retrieving values: get_option( 'your_field_id' )
class KWHero_Settings_Page {

    public function __construct() {
        add_action( 'admin_menu', array( $this, 'kwh_create_settings' ) );
        add_action( 'admin_init', array( $this, 'kwh_setup_sections' ) );
        add_action( 'admin_init', array( $this, 'kwh_setup_fields' ) );
    }

    public function kwh_create_settings() {
        $page_title = 'KWhero Settings ';
        $menu_title = 'KWHero';
        $capability = 'manage_options';
        $slug = 'KWHero';
        $callback = array($this, 'kwh_settings_content');
        add_options_page($page_title, $menu_title, $capability, $slug, $callback);
        
    }
    
    public function kwh_settings_content() { ?>
        <div class="wrap">
            <h1>KWHero Settings</h1>
            <p>Add your KWHero API key below. If you do not have one, you can get it from you KWHero Profile.</p>
            <?php settings_errors(); ?>
            <form method="POST" action="options.php" id="kwhero_settings_form">
                <?php
                    settings_fields( 'KWHero' );
                    do_settings_sections( 'KWHero' );
                    submit_button();
                ?>
            </form>
            <div id="saveResult"></div>
            <script type="text/javascript">
            jQuery(document).ready(function() {
                jQuery('#kwhero_settings_form').submit(function() { 
                    let email = jQuery('#kwhero_email').val();
                    let apiKey = jQuery('#kwhero_api_key').val();
                    //console.log(email + ' and ' + license);
                    var settings = {
                        "url": "https://app-be.kwhero.com/api/wp/users/is-valid",
                        "method": "POST",
                        "timeout": 0,
                        "headers": {
                            "Content-Type": "application/json"
                        },
                        "data": JSON.stringify({
                            "email": email,
                            "apiKey": apiKey
                        }),
                    };

                    jQuery.ajax(settings).done(function (response) {
                        try {
                        if(response.results){
                            jQuery('#kwhero_settings_form').ajaxSubmit({
                                success: function(){
                                    jQuery('#saveResult').html("<div id='saveMessage' class='successModal'></div>");
                                    jQuery('#saveMessage').append("<p><?php echo htmlentities(__('Settings Saved Successfully','wp'),ENT_QUOTES); ?></p>").show();
                                    console.log('success box');
                                },
                                error: function(){
                                    jQuery('#saveResult').html("<div id='saveMessage' class='errorModal'></div>");
                                    jQuery('#saveMessage').append("<p><?php echo htmlentities(__('An error occured','wp'),ENT_QUOTES); ?></p>").show();
                                },
                                timeout: 5000
                            }); 
                         
                        } else{
                            jQuery('#saveResult').html("<div id='saveMessage' class='errorModal'></div>");
                            jQuery('#saveMessage').append("<p><?php echo htmlentities(__('We could not verify your email and API key!','wp'),ENT_QUOTES); ?></p>").show();
                        }
                    } catch (e) {
                        jQuery('#saveResult').html("<div id='saveMessage' class='errorModal'></div>");
                        jQuery('#saveMessage').append("<p><?php echo htmlentities(__('An error occured','wp'),ENT_QUOTES); ?></p>").show();
                    }
                        setTimeout("jQuery('#saveMessage').hide('slow');", 5000);
                    });

                    return false; // disable the default behaviour of form submit (page relaod)
                    
                });
            });
            </script>
            <style>
                #saveMessage {
                    display: block;
                    position: fixed;
                    top: 50px;
                    right: 20px;
                    width: 300px;
                    height: auto;
                    padding: 5px 20px;
                    z-index:1002;
                    overflow: auto;
                    -moz-border-radius: 15px; 
                    -webkit-border-radius: 15px;
                    -moz-box-shadow: 5px 5px 10px #cfcfcf;
                    -webkit-box-shadow: 5px 5px 10px #cfcfcf;
                }
                .successModal {
                    border: 3px solid green;
                    background-color: #EFE;
                }
                .errorModal {
                    border: 3px solid red;
                    background-color: #ff7f7f;
                    color: #fff;
                }
            </style>
        </div> <?php
    }

    public function kwh_setup_sections() {
        add_settings_section( 'KWHero_section', 'Your KWHero account API settings', array(), 'KWHero' );
    }

    public function kwh_setup_fields() {
        $fields = array(
                    array(
                        'section' => 'KWHero_section',
                        'label' => 'Email',
                        'placeholder' => 'name@gmail.com',
                        'id' => 'kwhero_email',
                        'type' => 'email',
                    ),
        
                    array(
                        'section' => 'KWHero_section',
                        'label' => 'API key',
                        'placeholder' => 'xxxxxxxxxxxxxxxxxx',
                        'id' => 'kwhero_api_key',
                        'type' => 'password',
                    )
        );
        foreach( $fields as $field ){
            add_settings_field( $field['id'], $field['label'], array( $this, 'kwh_field_callback' ), 'KWHero', $field['section'], $field );
            register_setting( 'KWHero', $field['id'] );
        }
    }
    public function kwh_field_callback( $field ) {
        $value = get_option( $field['id'] );
        $placeholder = '';
        if ( isset($field['placeholder']) ) {
            $placeholder = $field['placeholder'];
        }
        switch ( $field['type'] ) {
            
            
            default:
                printf( '<input name="%1$s" id="%1$s" type="%2$s" placeholder="%3$s" value="%4$s" />',
                    $field['id'],
                    $field['type'],
                    $placeholder,
                    $value
                );
        }
        if( isset($field['desc']) ) {
            if( $desc = $field['desc'] ) {
                printf( '<p class="description">%s </p>', $desc );
            }
        }
    }
    
}
new KWHero_Settings_Page();

function KWHero_settings_scripts() {
    if ( is_admin() ){ // for Admin Dashboard Only
       // Embed the Script on our Plugin's Option Page Only
       if ( isset($_GET['page']) && $_GET['page'] == 'KWHero' ) {
          wp_enqueue_script('jquery');
          wp_enqueue_script( 'jquery-form' );
       }
    }
 }
add_action( 'admin_init', 'KWHero_settings_scripts' );



// alex added to check validity
add_action( 'rest_api_init', function () {
    register_rest_route( 'kwhero/v1', 'plugin-valid', array(
        'methods' => 'POST',
        'callback' => 'kwh_plugin_valid',
    ) );
  });

// check if API key from request and saved one are matched
function kwh_plugin_valid(WP_REST_Request $request) {
    // get stored options
    $savedApiKey = get_option( 'kwhero_api_key' );
    $requestApiKey = $request->get_param('apiKey');

    // Create the response object
    $response = new WP_REST_Response();
    
    // check if options are matched
    if ($savedApiKey == $requestApiKey) {
      $response->set_data( array( 'valid' => true) );
    } else {
      $response->set_data( array( 'valid' => false) );
    }
    
    return $response;
}