<?php

/***
 * update work 
 ***/

if( ! class_exists( 'KWHeroUpdateChecker' ) ) {

    class KWHeroUpdateChecker {

        public $plugin_slug;
        public $version;
        public $cache_key;
        public $cache_allowed;

        public function __construct() {
            $this->plugin_slug = 'kwhero-wordpress-integration/kwhero-wordpress-integration.php'; // Adjust to your plugin slug
            $this->version = KWH_CURRENT_VERSION; // Current version of your plugin
            $this->cache_key = 'kwhero_update_checker';
            $this->cache_allowed = true; // Change based on your caching preference

            add_filter( 'plugins_api', array( $this, 'info' ), 20, 3 );
            add_filter( 'site_transient_update_plugins', array( $this, 'update' ) );
            add_action( 'upgrader_process_complete', array( $this, 'purge' ), 10, 2 );
        }

        public function request() {
            $remote = get_transient( $this->cache_key );
            if( false === $remote || ! $this->cache_allowed ) {
                // Change the URL to your update server's endpoint
                $remote = wp_remote_get(
                    KWH_API_BASE.'plugin-updates',
                    array(
                        'timeout' => 10,
                        'headers' => array(
                            'Accept' => 'application/json'
                        )
                    )
                );

                if(
                    is_wp_error( $remote )
                    || 200 !== wp_remote_retrieve_response_code( $remote )
                    || empty( wp_remote_retrieve_body( $remote ) )
                ) {
                    return false;
                }

                set_transient( $this->cache_key, $remote, DAY_IN_SECONDS );
            }

            $remote = json_decode( wp_remote_retrieve_body( $remote ) );

            return $remote;
        }

        function info($res, $action, $args) {
            //delete_transient($cache_key);
        
            if ('plugin_information' !== $action) {
                return $res;
            }
        
            if ($this->plugin_slug !== $args->slug) {
                return $res;
            }
        
            $remote = $this->request();
        
            if (!$remote) {
                return $res;
            }
        
            // Construct a new stdClass object for the response
            $res = new stdClass();
        
            $res->name = $remote->name;
            $res->slug = $remote->slug;
            $res->version = $remote->version;
            $res->tested = $remote->tested;
            $res->requires = $remote->requires;
            $res->author = $remote->author;
            $res->author_profile = $remote->author_profile ?? ''; // Assuming there's an author_profile field
            $res->download_link = $remote->download_url;
            $res->requires_php = $remote->requires_php;
            $res->last_updated = $remote->last_updated;
        
            // Make sure sections are set as an associative array
            $res->sections = array(
                'description' => $remote->sections->description ?? '',
                'installation' => $remote->sections->installation ?? '',
                'configuration' => $remote->sections->configuration ?? '',
                'changelog' => $remote->sections->changelog ?? ''
            );
        
            // Banners should be an associative array if they exist
            if (isset($remote->banners) && is_object($remote->banners)) {
                $res->banners = array(
                    'low' => $remote->banners->low ?? '',
                    'high' => $remote->banners->high ?? ''
                );
            }
        
            return $res;
        }
        

        public function update( $transient ) {
            if ( empty($transient->checked) ) {
                return $transient;
            }
        
            $remote = $this->request();
            //my_plugin_log( 'this-ver'.$this->version );
            //my_plugin_log(('remote: '.print_r($remote, true)));
            if (
                $remote
                && version_compare($this->version, $remote->version, '<')
                && version_compare($remote->requires, get_bloginfo('version'), '<=')
                && version_compare($remote->requires_php, PHP_VERSION, '<=')
            ) {
                $res = new stdClass();
                $res->slug = $this->plugin_slug;
                // Make sure this is correctly pointing to your plugin file, usually it's like 'plugin-directory/plugin-file.php'
                $res->plugin = plugin_basename($this->plugin_slug); 
                $res->new_version = $remote->version;
                $res->tested = $remote->tested;
                $res->package = $remote->download_url; // Use the correct URL to the update package
        
                $transient->response[$res->plugin] = $res;
            }
        
            return $transient;
        }
        
        

        public function purge( $upgrader, $options ) {
            if (
                $this->cache_allowed
                && 'update' === $options['action']
                && 'plugin' === $options['type']
            ) {
                delete_transient( $this->cache_key );
            }
        }

    }

    new KWHeroUpdateChecker();
}


